package com.gic.store.service.outer;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.GlobalInfo;
import com.gic.store.constant.Constants;
import com.gic.store.constant.CreateTypeEnum;
import com.gic.store.dto.*;
import com.gic.store.entity.TabStoreBrand;
import com.gic.store.entity.TabStoreGroup;
import com.gic.store.entity.TabStoreTmp;
import com.gic.store.service.*;
import com.google.common.base.Joiner;
import com.task.allocation.api.AbstractTaskAllocationOperation;
import com.task.allocation.exception.TaskAllocationException;
import com.task.allocation.qo.InitTaskQo;
import com.task.allocation.util.TaskAllocationSdkClient;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * @author zhiwj
 * @date 2019/7/4
 */
@Service("storeTaskApiService")
public class StoreTaskServiceImpl extends AbstractTaskAllocationOperation implements StoreTaskApiService {

    private static final Logger logger = LogManager.getLogger(StoreTaskServiceImpl.class);

    @Autowired
    private StoreImportService storeImportService;

    @Autowired
    private StoreBrandService storeBrandService;

    @Autowired
    private ProvincesApiService provincesApiService;

    @Autowired
    private StoreDictApiService storeDictApiService;

    @Autowired
    private StoreGroupService storeGroupService;

    @Autowired
    private StoreApiService storeApiService;

    @Autowired
    private StoreService storeService;

    @Override
    public List<Object> getListTasks(String params) {
        // TODO
        List<TabStoreTmp> list = storeImportService.listUnImportStore(params);
        List<Object> listTask = new ArrayList<Object>(list);
        logger.info("门店批量导入放入队列数据数量：{}", list.size());
        return listTask;
    }

    @Override
    public void dealSingle(Object o) throws TaskAllocationException {
        StoreTmpDTO t = (StoreTmpDTO) o;
        try {
            ServiceResponse<Integer> serviceResponse = this.importDataToStore(t);
            if (!serviceResponse.isSuccess()) {
                t.setErrorMessage(serviceResponse.getMessage());
            }
        } catch (Exception e) {
            logger.warn("写入门店表和成员表异常", e);
            t.setErrorMessage("导入异常");
            storeImportService.updateData(t);
        }
    }

    public ServiceResponse<Integer> importDataToStore(StoreTmpDTO t) {
        StoreDTO storeDTO = new StoreDTO();
        storeDTO.setEnterpriseId(t.getEnterpriseId());
        storeDTO.setStoreName(t.getStoreName());

        if (storeService.countByStoreCode(t.getEnterpriseId(), t.getStoreCode(), null) > 0) {
            t.setErrorMessage("门店code已经存在");
            storeImportService.updateData(t);
        }
        storeDTO.setStoreCode(t.getStoreCode());
        storeDTO.setRegionId(t.getRegionId());
        storeDTO.setConactsPhone(t.getConactsPhone());
        storeDTO.setAddress(t.getAddress());

        String brands = t.getBrands();
        String[] brandArr = brands.split("、");
        List<TabStoreBrand> storeBrandList = storeBrandService.listAllStoreBrand(t.getEnterpriseId(), null, null, null);
        List<Integer> list = new ArrayList<>();
        for (String brandName : brandArr) {
            for (TabStoreBrand storeBrand : storeBrandList) {
                if (storeBrand.getStoreBrandName().equals(brandName)) {
                    list.add(storeBrand.getStoreBrandId());
                    break;
                }
            }
        }
        storeDTO.setBrandIds(Joiner.on(GlobalInfo.FLAG_COMMA).join(list));
        ServiceResponse<List<ProvinceDTO>> provinceResponse = provincesApiService.selectAllProvince();
        ServiceResponse<List<CityDTO>> cityResponse = provincesApiService.selectAllCity();
        ServiceResponse<List<CountyDTO>> countyResponse = provincesApiService.selectAllCounty();
        if (provinceResponse.isSuccess() && cityResponse.isSuccess() && countyResponse.isSuccess()) {
            List<ProvinceDTO> provinceDTOList = provinceResponse.getResult();
            Optional<ProvinceDTO> provinceDTO = provinceDTOList.stream().filter(e -> StringUtils.equals(e.getProvinceName(), t.getProvince())).findFirst();
            storeDTO.setProvinceId(provinceDTO.get().getProvinceId());
            List<CityDTO> cityDTOList = cityResponse.getResult();
            Optional<CityDTO> cityDTO = cityDTOList.stream().filter(e -> StringUtils.equals(e.getCityName(), t.getCity())).findFirst();
            storeDTO.setCityId(cityDTO.get().getCityId());
            List<CountyDTO> countyDTOList = countyResponse.getResult();
            Optional<CountyDTO> countyDTO = countyDTOList.stream().filter(e -> StringUtils.equals(e.getCountyName(), t.getCounty())).findFirst();
            storeDTO.setAreaId(countyDTO.get().getCountyId());
            storeDTO.setProvinces(provinceDTO.get().getProvinceName() + "\\" + cityDTO.get().getCityName() + "\\" + countyDTO.get().getCountyName());
        } else {
            logger.warn("省市区调用失败");
        }

        storeDTO.setLongitude(t.getLongitude());
        storeDTO.setLatitude(t.getLatitude());
        // todo 判断溢出
        storeDTO.setOverflowStatus(0);
        storeDTO.setStatus(2);

        ServiceResponse<List<StoreDictDTO>> erpStatusResponse = storeDictApiService.listStoreStatus(t.getEnterpriseId());
        if (erpStatusResponse.isSuccess()) {
            List<StoreDictDTO> erpList = erpStatusResponse.getResult();
            final String erpStatusTmp = t.getErpStatus().replace("\\s+", "");
            Optional<StoreDictDTO> erpStatusDTO = erpList.stream().filter(e -> e.getKey().equals(erpStatusTmp)).findFirst();
            storeDTO.setErpStatus(Integer.parseInt(erpStatusDTO.get().getValue()));
        } else {
            logger.warn("字典模块调用失败");
        }

        ServiceResponse<List<StoreDictDTO>> storeTypeResponse = storeDictApiService.listStoreType(t.getEnterpriseId());
        if (storeTypeResponse.isSuccess()) {
            List<StoreDictDTO> typeList = storeTypeResponse.getResult();
            final String typeTmp = t.getStoreType().replace("\\s+", "");
            Optional<StoreDictDTO> typeTmpDTO = typeList.stream().filter(e -> e.getKey().equals(typeTmp)).findFirst();
            storeDTO.setStoreType(Integer.parseInt(typeTmpDTO.get().getValue()));
        } else {
            logger.warn("字典模块调用失败");
        }

        TabStoreGroup group = storeGroupService.getStoreGroupByName(t.getStoreGroupName(), t.getEnterpriseId());
        storeDTO.setStoreGroupId(group.getStoreGroupId());

        storeDTO.setCreateTime(new Date());

        storeDTO.setCreateType(CreateTypeEnum.BATCH_IMPORT.getCode());
        // todo 扩展信息
        String customField = t.getCustomField();
        List<StoreExtendDTO> extendList = JSON.parseArray(customField, StoreExtendDTO.class);
        storeDTO.setStoreExtendList(extendList);

        return storeApiService.saveOrUpdate(storeDTO);
    }

    @Override
    public String importDataToStore(Integer enterpriseId, String userId, String uuId) {
        // TODO 队列
        String message = "";
        InitTaskQo initTaskQo = new InitTaskQo();
        initTaskQo.setEnterpriseId(enterpriseId.toString());// 企业id
        initTaskQo.setOperationUserId(userId);// 操作人id
        initTaskQo.setTaskMqKey(Constants.STORE_BATCH_IMPORT_MQ_KEY);// 注册的队列名称
        initTaskQo.setTaskType(Constants.STORE_BATCH_IMPORT_TASK_TYPE);// 任务类型
        initTaskQo.setParams(enterpriseId.toString());// 你自己处理需要的参数
        initTaskQo.setTaskSignKey(uuId);// 任务类型 唯一标识符
        try {
            TaskAllocationSdkClient instance = TaskAllocationSdkClient.getInstance();// 初始化sdk
            instance.initTask(initTaskQo);// 初始化接口
            logger.info("门店批量导入队列初始化成功");
        } catch (Exception e) {
            logger.warn("门店批量导入队列初始化异常", e);
            message = "门店批量导入队列初始化异常";
        }
        return message;
    }
}
