package com.gic.store.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.store.constant.StoreImportEnum;
import com.gic.store.dto.*;
import com.gic.store.service.*;
import com.gic.store.utils.ErrorCode;
import com.gic.store.web.ExcelUtils;
import com.gic.store.web.qo.PageQO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author zhiwj
 * @date 2019/7/3
 */
@RestController
public class StoreImportController {
    private static final Logger logger = LogManager.getLogger(StoreController.class);

    @Autowired
    private StoreApiService storeApiService;

    @Autowired
    private StoreImportApiService storeImportApiService;

    @Autowired
    private StoreRegionApiService storeRegionApiService;

    @Autowired
    private StoreDictApiService storeDictApiService;

    @Autowired
    private ProvincesApiService provincesApiService;

    @Autowired
    private StoreFieldApiService storeFieldApiService;

    /**
     * 下载模板
     * @param request
     * @param response
     * @return
     * @throws Exception
     */
    @RequestMapping("/store-import-template-download")
    public Object download(HttpServletRequest request, HttpServletResponse response) throws Exception {
//        ClassPathResource res = new ClassPathResource("门店资料导入模板.xlsx");
        String regionIdStr = request.getParameter("regionId");
        if (StringUtils.isBlank(regionIdStr)) {
            return RestResponse.failure(ErrorCode.ERR_5.getCode(), "请选择门店域");
        }
        Integer enterpriseId = 1111;
        List<List<String>> list = new ArrayList<>();

        ServiceResponse<List<List<String>>> titleServiceResponse = storeApiService.listImportTitle(enterpriseId, Integer.parseInt(regionIdStr));
        // 测试数据
        List<String> list3 = new ArrayList<>();
        Collections.addAll(list3, "1", "杭州测试总店", "A001", "自营", "上线", "12345678912", "杭州门店", "品牌1、品牌2", "浙江省", "杭州市", "西湖区", "华星时代广场", "经度", "纬度");
        Collections.addAll(list, list3);
        int size = list3.size();
        for (int i = 0; i < titleServiceResponse.getResult().get(0).size() - size; i++) {
            list3.add(" ");
        }
        Workbook workbook = ExcelUtils.getWorkbook("门店导入模板", titleServiceResponse.getResult().get(0), titleServiceResponse.getResult().get(1), list, regionIdStr);

        String fileName = "门店资料导入模板.xlsx";
        byte[] bytesName = fileName.getBytes("UTF-8");
        fileName = new String(bytesName, "ISO-8859-1");
        response.setHeader("content-disposition", "attachment;fileName=" + fileName);

        OutputStream out = response.getOutputStream();
        workbook.write(out);
        out.close();

        return RestResponse.success();
    }

    /**
     * 批量导入
     * @param request
     * @param file
     * @param pageQO
     * @return
     * @throws IOException
     */
    @RequestMapping("/store-tmp-file-upload")
    public RestResponse fileUpload(HttpServletRequest request, MultipartFile file, PageQO pageQO) throws IOException {
        Integer enterpriseId = 1111;
        if (file == null || file.isEmpty()) {
            return RestResponse.failure(ErrorCode.ERR_5.getCode(), "文件不存在");
        }
        // 判断是否有正在导入的任务
        ServiceResponse<Boolean> importingStoreResponse = storeImportApiService.hasImportingStore(enterpriseId);
        if (!importingStoreResponse.isSuccess()) {
            logger.info("门店导入模块调用失败");
            RestResponse.failure(ErrorCode.ERR_5.getCode(), "网络错误");
        }
        if (importingStoreResponse.getResult()) {
            return RestResponse.failure(ErrorCode.ERR_7.getCode(), "当前企业下有正在导入门店的任务，请稍后重试！");
        }
        if (file.getSize() > 0) {
            String fileName = file.getOriginalFilename();
            String uuId = request.getParameter("uuId");
            if (fileName.endsWith("xlsx")) {
                try {
                    XSSFWorkbook workbook = ExcelUtils.getWorkbookFromUpload(file.getInputStream(), fileName);
                    StoreRegionDTO storeRegion = this.readStoreRegion(enterpriseId, workbook);
                    if (storeRegion == null) {
                        return RestResponse.failure(ErrorCode.ERR_5.getCode(), "门店域不存在");
                    }
                    List<Map<Integer, String>> data = ExcelUtils.readExcel(6, fileName.substring(fileName.lastIndexOf(".") + 1), workbook);
                    ServiceResponse<List<List<String>>> titleServiceResponse = storeApiService.listImportTitle(enterpriseId, storeRegion.getRegionId());
                    List<String> titleList = titleServiceResponse.getResult().get(0);
                    // todo 这里在测试完成后放开
//                    if (CollectionUtils.isNotEmpty(data) && titleList.size() != data.get(0).size()) {
//                        return RestResponse.failure(ErrorCode.ERR_7.getCode(), "总列数与模板不一致");
//                    }
                    storeImportApiService.delStoreTmp(enterpriseId);
                    for (Map<Integer, String> row : data) {
                        StoreTmpDTO bean = this.validateData(row, enterpriseId, storeRegion.getRegionId());
                        bean.setRegionId(storeRegion.getRegionId());
                        bean.setRegionName(storeRegion.getRegionName());
                        try {
                            bean.setSignKey(uuId);
                            // 保存数据到临时表
                            storeImportApiService.save(bean);
                        } catch (Exception e) {
                            logger.warn(e);
                        }
                    }

                    try {
                        if (storeImportApiService.hasImportingStore(enterpriseId).getResult()) {
                            // TODO userId
                            String r = storeImportApiService.importDataToStore(enterpriseId, "123", uuId).getResult();
                            if (StringUtils.isNotBlank(r)) {
                                logger.warn("队列错误, {}", r);
                                return RestResponse.failure(ErrorCode.ERR_5.getCode(), "网络错误");
                            }
                        } else {
                            Page page = storeImportApiService.listStoreTmp(enterpriseId, false, pageQO.getCurrentPage(), pageQO.getPageSize()).getResult();
                            Map<Object, Object> result = new HashMap<>(8);
                            result.put("errorData", page);
                            int errorRows = page.getTotalCount();
                            result.put("total", errorRows);
                            result.put("success", 0);
                            result.put("error", errorRows);
                            return RestResponse.success(result);
                        }
                    } catch (Exception e) {
                        logger.warn(e);
                        return RestResponse.failure(ErrorCode.ERR_5.getCode(), "异步队列调用失败");
                    }
                } catch (Exception e) {
                    logger.warn(e);
                    return RestResponse.failure(ErrorCode.ERR_5.getCode(), "excel验证不通过");
                }
            }
        } else {
            return RestResponse.failure(ErrorCode.ERR_5.getCode(), "选择excel格式文件上传");
        }
        return RestResponse.success();
    }

    /**
     * 错误日志查看
     * @param page
     * @return
     */
    @RequestMapping("/store-import-load-error-data")
    public Object loadErrorData(PageQO page) {
        Integer enterpriseId = 1111;
        Map<String, Object> result = new HashMap<String, Object>();
        List<StoreTmpDTO> successList = storeImportApiService.listStoreTmp(enterpriseId, true).getResult();
        ServiceResponse<Page> serviceResponse = storeImportApiService.pageError(enterpriseId, page.getCurrentPage(), page.getPageSize());
        List<StoreTmpDTO> totalList = storeImportApiService.listAllStoreTmp(enterpriseId).getResult();
        Page pageResponse = serviceResponse.getResult();
        result.put("errorData", pageResponse);
        result.put("total", totalList == null ? 0 : totalList.size());
        result.put("success", successList == null ? 0 : successList.size());
        result.put("error", pageResponse.getResult().size());
        return RestResponse.success(result);
    }

    /**
     * 导出错误日志
     * @param request
     * @param response
     * @return
     * @throws IOException
     */
    @RequestMapping("/export-error-data")
    public RestResponse exportErrorData(HttpServletRequest request, HttpServletResponse response) throws IOException {
        Integer enterpriseId = 1111;
        String fileName = "门店导入错误记录.xlsx";
        List<StoreTmpDTO> listErrorData = storeImportApiService.listStoreTmp(enterpriseId, false).getResult();
        if (!listErrorData.isEmpty()) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmm");
            fileName = sdf.format(listErrorData.get(0).getUpdateTime()) + fileName;
            List<List<String>> errorList = storeImportApiService.listError(enterpriseId).getResult();
            List<String> titleList = errorList.get(0);
            errorList.remove(0);
            Workbook workbook = ExcelUtils.getWorkbook("门店导入模板", titleList, null, errorList, null);
            byte[] bytesName = fileName.getBytes("UTF-8");
            fileName = new String(bytesName, "ISO-8859-1");
            response.setHeader("content-disposition", "attachment;fileName=" + fileName);

            OutputStream out = response.getOutputStream();
            workbook.write(out);
            out.close();
            return RestResponse.success();
        } else {
            return RestResponse.failure(ErrorCode.ERR_3.getCode(), "没有错误日志");
        }
    }

    private StoreTmpDTO validateData(Map<Integer, String> row, Integer enterpriseId, Integer regionId) {
        boolean validate = true;
        StoreTmpDTO bean = new StoreTmpDTO();
        bean.setEnterpriseId(enterpriseId);
        bean.setErrorMessage(StoreImportEnum.WAIT.getCode());

        // 验证门店名称
        String storeName = row.get(1);
        bean.setStoreName(storeName);
        if (StringUtils.isBlank(storeName)) {
            bean.setErrorMessage("门店名称不能为空");
            validate = false;
        }
        if (validate) {
            if (this.lengthValidate(storeName, 20)) {
                bean.setErrorMessage("门店名称长度超过20个字");
                validate = false;
            }
        }

        // 验证门店代码
        String storeCode = row.get(2);
        bean.setStoreCode(storeCode);
        if (validate) {
            if (StringUtils.isBlank(storeCode)) {
                bean.setErrorMessage("门店代码不能为空");
                validate = false;
            }
        }
        if (validate) {
            if (this.lengthValidate(storeCode, 20)) {
                bean.setErrorMessage("门店代码长度超过20个字");
                validate = false;
            }
        }
        if (validate) {
            // 验证门店code唯一
            if (storeImportApiService.validateStoreCode(enterpriseId, storeCode).getResult()) {
                bean.setErrorMessage("门店编码重复");
                validate = false;
            }
        }

        // 验证门店类型
        String storeType = row.get(3);
        bean.setStoreType(storeCode);
        if (validate) {
            if (StringUtils.isBlank(storeCode)) {
                bean.setErrorMessage("门店类型不能为空");
                validate = false;
            }
        }
        // todo
//        if (validate) {
//            if (this.storeTypeValidate(enterpriseId, storeType) == -1) {
//                bean.setErrorMessage("门店类型不存在");
//                validate = false;
//            }
//        }
        // 验证门店状态
        String erpStatus = row.get(4);
        bean.setErpStatus(erpStatus);
        if (validate) {
            if (StringUtils.isBlank(erpStatus)) {
                bean.setErrorMessage("门店状态不能为空");
                validate = false;
            }
        }
        //todo
//        if (validate) {
//            if (this.erpStatusValidate(enterpriseId, erpStatus) == -1) {
//                bean.setErrorMessage("门店状态不存在");
//                validate = false;
//            }
//        }
        // 验证门店电话
        String conactsPhone = row.get(5);
        bean.setConactsPhone(conactsPhone);
        if (validate) {
            if (StringUtils.isBlank(conactsPhone)) {
                bean.setErrorMessage("门店电话不能为空");
                validate = false;
            }
        }
        // 验证门店所属分组
        String storeGroupName = row.get(6);
        if (StringUtils.isBlank(storeGroupName)) {
            validate = false;
            bean.setErrorMessage("门店分组不能为空");
        }
        if (validate) {
            if (StringUtils.isBlank(storeImportApiService.isExistStoreGroup(enterpriseId, storeGroupName).getResult())) {
                validate = false;
                bean.setErrorMessage("门店分组不存在");
            }
        }
        bean.setStoreGroupName(storeGroupName);

        // 验证品牌
        String brands = row.get(7);
        bean.setBrands(brands);
        if (validate) {
            String errorMsg = brandsValidate(enterpriseId, brands);
            if (errorMsg != null) {
                bean.setErrorMessage(errorMsg);
                validate = false;
            }
        }

        // 验证省份
        String province = row.get(8);
        bean.setProvince(province);
        if (validate) {
            if (StringUtils.isBlank(province)) {
                bean.setErrorMessage("门店省份不能为空");
                validate = false;
            }
        }
        // 验证市
        String city = row.get(9);
        bean.setCity(city);
        if (validate) {
            if (StringUtils.isBlank(city)) {
                bean.setErrorMessage("门店市不能为空");
                validate = false;
            }
        }
        // 验证区
        String county = row.get(10);
        bean.setCounty(county);
        if (validate) {
            if (StringUtils.isBlank(county)) {
                bean.setErrorMessage("门店区县不能为空");
                validate = false;
            }
        }
        // 验证省市区数据的正确性
        if (validate) {
            ServiceResponse<List<ProvinceDTO>> provinceResponse = provincesApiService.selectAllProvince();
            ServiceResponse<List<CityDTO>> cityResponse = provincesApiService.selectAllCity();
            ServiceResponse<List<CountyDTO>> countyResponse = provincesApiService.selectAllCounty();
            if (provinceResponse.isSuccess() && cityResponse.isSuccess() && countyResponse.isSuccess()) {
                List<ProvinceDTO> provinceDTOList = provinceResponse.getResult();
                long provinceCount = provinceDTOList.stream().filter(e -> StringUtils.equals(e.getProvinceName(), province)).count();
                List<CityDTO> cityDTOList = cityResponse.getResult();
                long cityCount = cityDTOList.stream().filter(e -> StringUtils.equals(e.getCityName(), city)).count();
                List<CountyDTO> countyDTOList = countyResponse.getResult();
                long countyCount = countyDTOList.stream().filter(e -> StringUtils.equals(e.getCountyName(), county)).count();
                if (countyCount == 0) {
                    bean.setErrorMessage("输入的区县信息有误");
                }
                if (cityCount == 0) {
                    bean.setErrorMessage("输入的市信息有误");
                }
                if (provinceCount == 0) {
                    bean.setErrorMessage("输入的省信息有误");
                }
            } else {
                logger.warn("省市区调用服务失败");
            }
        }
        // 验证门店详细地址
        String address = row.get(11);
        bean.setAddress(address);
        if (validate) {
            if (StringUtils.isBlank(address)) {
                bean.setErrorMessage("门店详细地址不能为空");
                validate = false;
            }
        }
        bean.setLongitude(row.get(12));
        bean.setLatitude(row.get(13));

        // 自定义属性
        //todo 自定义属性验证
        ServiceResponse<List<StoreFieldDTO>> fieldServiceResponse = storeFieldApiService.listStoreFieldByRegionId(regionId);
        List<StoreFieldDTO> fieldList = fieldServiceResponse.getResult();
        List<StoreExtendDTO> extendDTOList = new ArrayList<>();
        for (int i = 0; i < fieldList.size(); i++) {
            StoreExtendDTO storeExtendDTO = new StoreExtendDTO();
            storeExtendDTO.setStoreFieldId(fieldList.get(i).getStoreFieldId());
            storeExtendDTO.setValue(row.get(14 + i));
            storeExtendDTO.setSort(fieldList.get(i).getSort());
            // 验证
            if (validate) {
//                String s = fieldValidate(storeExtendDTO.getValue(), fieldList.get(i).getStoreFieldDetail());
            }
            extendDTOList.add(storeExtendDTO);
        }
        bean.setCustomField(JSON.toJSONString(extendDTOList));
        return bean;
    }

    private String brandsValidate(Integer enterpriseId, String brands) {
        ServiceResponse<List<String>> response = storeImportApiService.listBrandByNotExist(enterpriseId, brands);
        if (response.isSuccess()) {
            List<String> brandList = response.getResult();
            if (CollectionUtils.isNotEmpty(brandList)) {
                String error = brandList.stream().reduce((x, y) -> x + " " + y).get();
                return "品牌 " + error + " 不存在";
            }
            return null;
        } else {
            return "系统错误";
        }
    }

    private int erpStatusValidate(Integer enterpriseId, String erpStatus) {
        ServiceResponse<List<StoreDictDTO>> serviceResponse = storeDictApiService.listStoreStatus(enterpriseId);
        if (serviceResponse.isSuccess()) {
            List<StoreDictDTO> list = serviceResponse.getResult();
            final String erpStatusTmp = erpStatus.replace("\\s+", "");
            long count = list.stream().filter(e -> e.getKey().equals(erpStatusTmp)).count();
            if (count > 0) {
                return (int) count;
            }
        }
        return -1;
    }

    private int storeTypeValidate(Integer enterpriseId, String storeType) {
        ServiceResponse<List<StoreDictDTO>> serviceResponse = storeDictApiService.listStoreType(enterpriseId);
        if (serviceResponse.isSuccess()) {
            List<StoreDictDTO> list = serviceResponse.getResult();
            final String storeTypeTmp = storeType.replace("\\s+", "");
            long count = list.stream().filter(e -> e.getKey().equals(storeTypeTmp)).count();
            if (count > 0) {
                return (int) count;
            }
        }
        return -1;
    }

    private boolean lengthValidate(String value, int len) {
        String _value = value.replaceAll("[^\\x00-\\xff]", "**");
        return _value.length() / 2.0 > (double) len;
    }

    private StoreRegionDTO readStoreRegion(Integer enterpriseId, XSSFWorkbook workbook) {
        XSSFSheet sheet = workbook.getSheetAt(0);
        XSSFRow row = sheet.getRow(0);
        XSSFCell cell = row.getCell(0);
        ServiceResponse<StoreRegionDTO> serviceResponse = storeRegionApiService.getStoreRegionByRegion(enterpriseId, (int) cell.getNumericCellValue());
        return serviceResponse.isSuccess() ? serviceResponse.getResult() : null;
    }


//    @RequestMapping("/test")
//    public Object re() {
//        List<StoreTmpDTO> storeTmpDTOS = storeImportApiService.listStoreTmp(1111, null, true);
//        System.out.println(storeTmpDTOS);
//        return null;
//    }

}
